package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// Abstract class for parse log files by lines or by sectors
//------------------------------------------------------------------------------

abstract class BLParser {
	
	BLParser() {}
	
	/**
	 * Parses rows from buildLog by rules from queue
	 *
	 * @param path - path to buildLog
	 * @param sectors - object, in which will be pushed each item from buildLog
	 *
	 * @return true, if was at least one ERROR inside buildLog. Otherwise, false.
	 */
	abstract public void parseByQueue(String path, def items);
	
	/**
	 * Creates body of HTML file, that contains references to each founded item
	 *
	 * @param errorSectors - map with founded items
	 *
	 * @return body
	 */
	abstract public String createRefFrame(Map errorLines);
	
	/**
	 * Creates body of HTML file, that contains each line from buildLog
	 *
	 * @param sectorsQueue - queue, that contains each sector from buildLog
	 *
	 * @return body
	 */
	abstract public String createContentFrame(def linesQueue);
	
	/**
	 * Return all error items, which were defined as ErrorFlag
	 * @return
	 */
	abstract public def getParsedMessagesByFlag();
	
	/**
	 * Return error items by error type
	 * @param type - error type
	 * @return
	 */
	abstract public def getParsedMessagesByType(String type);
	
	/**
	 * Parses and creates output HTML files.
	 *
	 * @param pathToBuildLog - path to buildLog
	 * @param outputFile - path to the output file
	 *
	 * @return false, if was at least one ERROR inside buildLog. Otherwise, true.
	 * @throws Exception
	 */
	abstract public boolean process(String pathToBuildLog, String outputFile);
	
	/**
	 * Create *.html output file
	 *
	 * @param outputFile - path to the output file
	 * @param refContent - references part
	 * @param logContent - content part
	 * @throws Exception
	 */
	public void createLogHtml(String outputFile, String refContent, String logContent) throws Exception{
		
			BufferedWriter bWriter = null;
			
			try{
				bWriter = new BufferedWriter(new FileWriter(outputFile));
				
				bWriter.write("""
					<html>
					<head>
					<script type="text/javascript">
						function toggleList(list){
							element = document.getElementById(list).style;
							element.display == 'none' ? element.display='block' : element.display='none';
						}
					</script>

					</head>
					<body>
					<div style="float:left; width:100%; height:100%;"> 
					<div style="float:left; width:25%; height:100%; overflow-y:scroll;">
				""");
				
				bWriter.write(refContent);
				
				bWriter.write("""
					</div>
					<div style="float:left; width:75%; height:100%; overflow-y:scroll;">
					<div style="padding-left: 1em; padding-right: 1em; white-space: pre-wrap;">""");
				
				bWriter.write(logContent);
				
				bWriter.write("""
					</div>
					</div>
					</div>
					</body>
					</html>
				""");
			
				bWriter.close();
				
			} catch(Exception e) {
				if(bWriter != null) {
					bWriter.close();
				}
				
				throw new Exception(e);
			}
			
			
	}

	/**
	 * Replace special characters
	 *
	 * @param line - text
	 * @return modified text
	 */
	public String replaceSpecialCharacters(String line){
		String text = line;

		Map specialCharacters = ["<": "&lt;", ">": "&gt;"]

		specialCharacters.each { key, value ->
			text = text.replace(key, value)
		}

		return text;
	}
}