package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// This library reads the  buildLog file and creates HTML page with link to each error, warning, info and so on.
// Error, warning, ..., ... should be define in XML file or can be used default values.
// For declaring regex rules should be use rules file, that contains list rules. Rules using from file in form queue.
// First rule from file will check firstly, second - secondly and so on.
//
// This parser can be used not only for parsing buildLog, but for parsing other text documents.
//------------------------------------------------------------------------------

import java.util.regex.Pattern;

import lib_groovy.Line;
import lib_groovy.Sector;
import lib_groovy.Rule;
import lib_groovy.RuleConfigReader;
import lib_groovy.Delimiter;
import lib_groovy.BLParser;

public class BuildLogParser {
	private RuleTypes types = null;
	private ArrayList<Rule> rulesQueue = null;
	private Sectors sectors = null;
	ArrayList<Delimiter> delimQueue = null;
	//parser
	BLParser parser;
	
	public BuildLogParser() {};
	
	/**
	 * Runs parsing and creates output HTML files.
	 * Works without configuration xml, because uses default xml inside self.
	 *
	 * @param pathToRules - path to the file with rules
	 * @param pathToBuildLog - path to buildLog
	 * @param outputFile - path to the output file
	 *
	 * @return false, if was at least one ERROR inside buildLog. Otherwise, true.
	 * @throws Exception
	 */
	public boolean run(String pathToRules, String pathToBuildLog, String outputFile) throws Exception{
		
		types = RuleConfigReader.readFromXml(new File(pathToRules).text);
		
		ArrayList<RuleType> sort = types.getSortedTypes();
		
		if(types == null || types.getSortedTypes().empty) {
			ZfReport.info("Started with default configuration file");
			
			String configFileText = """	<!-- <?xml version="1.0" encoding="UTF-8" standalone="no"?> -->
									<Configuration>
									<Types>
										<!-- -->
									    <Type name="ERROR"			color="red"			priority="1" 		errorFlag="true"/>
									    <!-- -->
									    <Type name="WARNING"		color="blue"		priority="2"/>
									    <!-- -->
									    <Type name="INFO"			color="silver"		priority="3"/>
									</Types>
									</Configuration>
								""";
								
			types = RuleConfigReader.readFromXml(configFileText);
		}
		
		
		boolean isError = true;
		
		
		isError = process(pathToRules, pathToBuildLog, outputFile);
	
		
		
		return isError;
	}
		
	
	
	/**
	 * Parses and creates output HTML files.
	 *
	 * @param pathToRules - path to the file with rules
	 * @param pathToBuildLog - path to buildLog
	 * @param outputFile - path to the output file
	 *
	 * @return false, if was at least one ERROR inside buildLog. Otherwise, true.
	 * @throws Exception
	 */
	private boolean process(String pathToRules, String pathToBuildLog, String outputFile)  throws Exception{
		
		ArrayList<RuleType> typesArray = types.getSortedTypes();
		
		ZfReport.info("Parsing rules...");
		RulesParser rulesParser = new RulesParser();
		rulesParser.parse(pathToRules, types);
		
		rulesQueue = rulesParser.getRules();
		
		delimQueue = rulesParser.getDelimiters();
		
		
		if(delimQueue.isEmpty()) {
			parser = new LinesParser();
			parser.setConfiguration(typesArray, rulesQueue);
		} else {
			parser = new SectorsParser();
			parser.setConfiguration(typesArray, rulesQueue, delimQueue);
		}
		
		
		
		boolean isFlag = parser.process(pathToBuildLog, outputFile);
		
		
		return isFlag;
	}
	
	
	public def getParsedMessagesByFlag() {
		parser.getParsedMessagesByFlag()
	}
	
	public def getParsedMessagesByType(String type) {
		parser.getParsedMessagesByType(type);
	}
	
}
