package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// Object, which will created on base this class, will contain all lines from buildLog
// in two structures: Queue with all lines and Map with founded items.
//------------------------------------------------------------------------------

import lib_groovy.Line
import lib_groovy.RuleTypes

class Lines{
	
	//Queue with all lines
	private ArrayList<Line> linesQueue = new ArrayList<Line>();
	//Map with founded items
	private Map errorLines = [:];
	
	/**
	 * Constructor creates Queue in Map for each type of rule
	 * 
	 * @param types - ArrayList with types of rules
	 */
	public Lines(ArrayList<RuleType> types) {
		for(RuleType ruleType: types) {
			ArrayList<Line> currQueue = new ArrayList<Line>();
			errorLines.put(ruleType, currQueue);
		}
	}
	
	/**
	 * Creates common line and adds to Queue
	 * 
	 * @param content - content of line
	 */
	public void addLine(String content) {
		Line line = new Line(content);
		
		linesQueue.add(line);
	}
	
	/**
	 * Creates line with founded item. Adds this line to Queue and Map.
	 * 
	 * @param content - content of line
	 * @param type - type of the item in the current line
	 * @param ref - reference to the current line
	 * @param lineNumber - line number from log file
	 */
	public void addLine(String content, RuleType type, String ref, int lineNumber) {
		Line line = new Line(content, type, ref, lineNumber);
		
		linesQueue.add(line);
		errorLines.getAt(type).add(line);
	}
	
	/**
	 * Checks if exist rules of this type
	 * 
	 * @param type - rule type
	 * 
	 * @return true if exist
	 */
	public boolean checkType(RuleType type) {
		return !errorLines.getAt(type).isEmpty();
	}
	
	/**
	 * Return all lines from build log
	 * @return
	 */
	public ArrayList<Line> getLinesQueue() {
		return linesQueue;
	}
	
	/**
	 * Return error map
	 * @return
	 */
	public Map getErrorLines() {
		return errorLines;
	}

	/**
	 * Return all error lines, which were defined as ErrorFlag
	 * @return
	 */
	public def getErrorLinesByFlag() {
		
		ArrayList<Line> errLines = new ArrayList<Line>();
		
		Iterator it = errorLines.iterator();
		
		while(it.hasNext()) {
			def curr = it.next();
			
			RuleType rType = curr.key;
			
			
			if(rType.isErrorFlag()) {
				def errors = curr.value;
				
				for(Line currLine: errors) {
					errLines.add(currLine);
				}
			}
		}
		
		return errLines;
	}
	
	/**
	 * Return error lines by error type
	 * @param type - error type
	 * @return
	 */
	public def getErrorsByType(String type){
		Iterator it = errorLines.iterator();
		
		while(it.hasNext()) {
			def curr = it.next();
			
			RuleType rType = curr.key;
			
			if(rType.getName().equalsIgnoreCase(type)) {
				def errors = curr.value;
				
				if(!errors.isEmpty()) {
					return errors;
				}
				
				return null;
			}
		}
		
		return null;
	}
	
	/**
	 * Checks, if error line define as error flag
	 * @return true if error flag
	 */
	public boolean isErrorFlag() {
		Iterator it = errorLines.iterator();
		
		while(it.hasNext()) {
			def curr = it.next();
			
			RuleType rType = curr.key;
			
			
			if(rType.isErrorFlag()) {
				def errors = curr.value;
				
				if(!errors.isEmpty()) {
					return true;
				}
			}
		}
		
		return false;
	}

	
	
}