package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
//	Parse log by lines
//------------------------------------------------------------------------------

import java.util.regex.Pattern;

import lib_groovy.Line;
import lib_groovy.Rule;
import lib_groovy.RuleConfigReader;

class LinesParser extends BLParser {
	
	private RuleTypes types = null;
	//list of lines rules
	private ArrayList<Rule> rulesQueue = null;
	private Lines lines = null;
	//list of rule types
	private ArrayList<RuleType> typesArray = null;
	
	public LinesParser() {};
	
	/**
	 * Set configuration
	 *
	 * @param typesArray - list of rule types
	 * @param rulesQueue - list of lines rules
	 */
	public void setConfiguration(ArrayList<RuleType> typesArray, ArrayList<Rule> rulesQueue) {
		this.typesArray = typesArray;
		this.rulesQueue = rulesQueue;
	}
	
	/**
	 * Parses and creates output HTML files. 
	 * 
	 * @param pathToBuildLog - path to buildLog
	 * @param outputFile - path to the output file
	 * 
	 * @return false, if was at least one ERROR inside buildLog. Otherwise, true.
	 * @throws Exception
	 */
	public boolean process(String pathToBuildLog, String outputFile){
		
		lines = new Lines(typesArray);
		
		ZfReport.info("Reading build log...");
		parseByQueue(pathToBuildLog, lines);
		
		ZfReport.info("Creating referencies part...");
		String refContent = createRefFrame(lines.getErrorLines());
		
		ZfReport.info("Creating content...");
		String logContent = createContentFrame(lines.getLinesQueue());
		
		ZfReport.info("Writing to the file...");
		createLogHtml(outputFile, refContent, logContent);
		
		ZfReport.info("File ${outputFile} created!");
		
		boolean isFlag = lines.isErrorFlag();
		
		
		return !isFlag;
	}
	
	/**
	 * Parses rows from buildLog by rules from queue
	 * 
	 * @param path - path to buildLog
	 * @param lines - object, in which will be pushed each row from buildLog
	 * 
	 * @return true, if was at least one ERROR inside buildLog. Otherwise, false.
	 */
	public void parseByQueue(String path, def lines) {
		File file = new File(path);
		
		int i = 0;
		
		file.withReader { reader ->
			
			String line;
			String fullLine;
			int lineNum = 0;
			
			while ((line = reader.readLine()) != null) {
				lineNum++;
				
				boolean isFound = false;
				fullLine = replaceSpecialCharacters(line);
				line = line.trim();
				
				def matcher = null;
				
				if(!line.empty) { 
				
					for(Rule rule: rulesQueue) {
						
						matcher = line =~ /${rule.getRegex()}/;
						
						if(matcher) {
							i++;
							
							String refToLine = rule.getType().getName().toUpperCase() + i;
							
							lines.addLine(fullLine, rule.getType(), refToLine, lineNum);
							
							//ZfReport.info("[!FOUND LINE!] Line " + lineNum + ": " + rule.getType().getName() + " " + rule.getRegex() + " Full line: " + fullLine);
							
							isFound = true;
							
							break;
						}
						
					}
				}
				
				if(!isFound) { 
					lines.addLine(fullLine);
				}
				
				
			}
			
		}
		
		ZfReport.info("[!FOUND LINES!] " + i + " lines found");
		
	}
	
	
	
	
	/**
	 * Creates body of HTML file, that contains references to each founded item
	 * 
	 * @param errorLines - map with founded items
	 * 
	 * @return body
	 */
	public String createRefFrame(Map errorLines) {
		Iterator it = errorLines.iterator();
		
		String body = "";
		
		while(it.hasNext()) {
			def curr = it.next();
			
			RuleType type = curr.key;
			
			String typeName = type.getName().toUpperCase();
			
			String color = type.getColor();
			
			def currLines = curr.value;
			
			
			String label = """<a href="javascript:toggleList('${typeName}')" target="_self"><strong>${typeName} (${currLines.size()})</strong></a><br>""";
			
			String ulLiPart ="""<ul id="${typeName}" style="display: none; margin-left: 0px; padding-left: 3em;">""";
			
			for(int i = 0; i < currLines.size(); i++) {
				Line line = currLines.getAt(i);
				
				String partForLi = line.getContent();
				
				if(partForLi.length() > 50) {
					partForLi = partForLi.substring(0, 50);
				}
				
				ulLiPart +=  """<li>${i+1} <a href="#${line.getRef()}"><span style="color:${color}">[${typeName}] ... ${partForLi} ...</span></a></li><br>"""
			
			}
			
			ulLiPart += "</ul>";
			body += label + ulLiPart;
		}
		
		return body;
	}
	
	
	/**
	 * Creates body of HTML file, that contains each line from buildLog
	 * 
	 * @param linesQueue - queue, that contains each line from buildLog
	 * 
	 * @return body
	 */
	public String createContentFrame(def linesQueue) {
		StringBuilder body = new StringBuilder("");
		
		for(int i = 0; i < linesQueue.size(); i++) {
			Line line = linesQueue.getAt(i);
			
			if(line.getType() == null) {
				body.append(line.getContent() + "\n");
			} else {
				RuleType type = line.getType();
				body.append("""<a name="${line.getRef()}"></a>""");
				body.append("""<span style="color:${type.getColor()}">[${type.getName().toUpperCase()} - ${i+1}] ${line.getContent()} </span>\n""");
			}
		}
		
		return body.toString();
	}
	
	
	
	/**
	 * Return all error lines, which were defined as ErrorFlag
	 * @return
	 */
	public def getParsedMessagesByFlag() {
		if(lines != null) {
			return lines.getErrorLinesByFlag();
		}
		
		return null;
	}
	
	/**
	 * Return error lines by error type
	 * @param type - error type
	 * @return
	 */
	public def getParsedMessagesByType(String type) {
		if(lines != null) {
			return lines.getErrorsByType(type);
		}
		
		return null;
	}
	
}