package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// Object, which will created on base this class, will contain all sectors from buildLog
// in two structures: Queue with all sectors and Map with founded items.
//------------------------------------------------------------------------------

class Sectors{
	//Queue with all sectors
	private ArrayList<Sector> sectorsQueue = new ArrayList<Sector>();
	//Map with founded items
	private Map errorSectors = [:];
	
	/**
	 * Constructor creates Queue in Map for each type of rule
	 *
	 * @param types - ArrayList with types of rules
	 */
	public Sectors(ArrayList<RuleType> types) {
		for(RuleType ruleType: types) {
			ArrayList<Sector> currQueue = new ArrayList<Sectors>();
			errorSectors.put(ruleType, currQueue);
		}
	}
	
	/**
	 * Creates common sector and adds to Queue
	 *
	 * @param content - content of line
	 * @param startLineNumber - sector start line number
	 * @param endLineNumber - sector end line number
	 */
	public void addSector(ArrayList<Line> linesQueue, int startLineNumber, int endLineNumber) {
		Sector sector = new Sector(linesQueue, startLineNumber, endLineNumber);
		
		sectorsQueue.add(sector);
	}
	
	/**
	 * Creates sector with founded item. Adds this sector to Queue and Map.
	 *
	 * @param linesQueue - content of sector
	 * @param type - type of the item in the current line
	 * @param ref - reference to the current line
	 * @param startLineNumber - sector start line number
	 * @param endLineNumber - sector end line number
	 */
	public void addSector(ArrayList<Line> linesQueue, RuleType type, String ref, int startLineNumber, int endLineNumber) {
		Sector sector = new Sector(linesQueue, type, ref, startLineNumber, endLineNumber);
		
		sectorsQueue.add(sector);
		errorSectors.getAt(type).add(sector);
	}
	
	
	/**
	 * Checks if exist rules of this type
	 *
	 * @param type - rule type
	 *
	 * @return true if exist
	 */
	public boolean checkType(RuleType type) {
		return !errorSectors.getAt(type).isEmpty();
	}
	
	/**
	 * Return all sectors from build log
	 * @return
	 */
	public ArrayList<Sector> getSectorsQueue() {
		return sectorsQueue;
	}
	
	/**
	* Return error map
	* @return
	*/
   public Map getErrorSectors() {
	   return errorSectors;
   }
   
   /**
	* Return all error sectors, which were defined as ErrorFlag
	* @return
	*/
   public def getErrorSectorsByFlag() {
	   
	   ArrayList<Line> errSectors = new ArrayList<Line>();
	   
	   Iterator it = errorSectors.iterator();
	   
	   while(it.hasNext()) {
		   def curr = it.next();
		   
		   RuleType rType = curr.key;
		   
		   
		   if(rType.isErrorFlag()) {
			   def errors = curr.value;
			   
			   for(Sector currSector: errors) {
				   Line line = new Line(currSector.getInfoWithoutRef(), currSector.getType(), currSector.getRef(), currSector.getStartLineNumber());
				   
				   errSectors.add(line);
			   }
		   }
	   }
	   
	   return errSectors;
   }
   
   /**
	* Return error sectors by error type
	* @param type - error type
	* @return
	*/
   public def getErrorsByType(String type){
	   Iterator it = errorSectors.iterator();
	   
	   while(it.hasNext()) {
		   def curr = it.next();
		   
		   RuleType rType = curr.key;
		   
		   if(rType.getName().equalsIgnoreCase(type)) {
			   def errors = curr.value;
			   
			   if(!errors.isEmpty()) {
				   ArrayList<Line> errLines = new ArrayList<Line>();
				   
				   for(Sector currSector: errors) {
					   
					   Line line = new Line(currSector.getInfoWithoutRef(), currSector.getType(), currSector.getRef(), currSector.getStartLineNumber());
					   
					   errLines.add(line);
				   }
				   
				   return errLines;
			   }
			   
			   return null;
		   }
	   }
	   
	   return null;
   }
   
   /**
	* Checks, if error sector define as error flag
	* @return true if error flag
	*/
   public boolean isErrorFlag() {
	   Iterator it = errorSectors.iterator();
	   
	   while(it.hasNext()) {
		   def curr = it.next();
		   
		   RuleType rType = curr.key;
		   
		   
		   if(rType.isErrorFlag()) {
			   def errors = curr.value;
			   
			   if(!errors.isEmpty()) {
				   return true;
			   }
		   }
	   }
	   
	   return false;
   }
}