package lib_groovy;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : build_log_parser Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// Parse log by sectors and lines together
//------------------------------------------------------------------------------

import java.util.regex.Pattern;

import lib_groovy.Line;
import lib_groovy.Sector;
import lib_groovy.Rule;
import lib_groovy.RuleConfigReader;
import lib_groovy.Delimiter;

public class SectorsParser  extends BLParser {
	
	private RuleTypes types = null;
	//list of lines rules
	private ArrayList<Rule> rulesQueue = null;
	private Sectors sectors = null;
	//list of delimiters
	private ArrayList<Delimiter> delimQueue = null;
	//list of rule types
	private ArrayList<RuleType> typesArray = null;
	
	/**
	 * Constructor 
	 * 
	 * @param delimiter - delimiter for sectors
	 */
	public SectorsParser() {};
	
	/**
	 * Set configuration
	 * 
	 * @param typesArray - list of rule types
	 * @param rulesQueue - list of lines rules
	 * @param delimQueue - list of delimiters
	 */
	public void setConfiguration(ArrayList<RuleType> typesArray, ArrayList<Rule> rulesQueue, ArrayList<Delimiter> delimQueue) {
		this.typesArray = typesArray;
		this.rulesQueue = rulesQueue;
		this.delimQueue = delimQueue;
	}
	
	/**
	 * Parses and creates output HTML files.
	 *
	 * @param pathToBuildLog - path to buildLog
	 * @param outputFile - path to the output file
	 *
	 * @return false, if was at least one ERROR inside buildLog. Otherwise, true.
	 * @throws Exception
	 */
	public boolean process(String pathToBuildLog, String outputFile){
		
		sectors = new Sectors(typesArray);
		
		ZfReport.info("Reading build log: " + pathToBuildLog);
		parseByQueue(pathToBuildLog, sectors);
		
		ZfReport.info("Creating referencies part...");
		String refContent = createRefFrame(sectors.getErrorSectors());
		
		ZfReport.info("Creating content...");
		String logContent = createContentFrame(sectors.getSectorsQueue());
		
		ZfReport.info("Writing to the file " + outputFile);
		createLogHtml(outputFile, refContent, logContent);
		
		ZfReport.info("File ${outputFile} created!");
		
		boolean isFlag = sectors.isErrorFlag();
		
		
		return !isFlag;
	}
	
	/**
	 * Parses rows from buildLog by rules from queue
	 *
	 * @param path - path to buildLog
	 * @param sectors - object, in which will be pushed each sector from buildLog
	 *
	 * @return true, if was at least one ERROR inside buildLog. Otherwise, false.
	 */
	public void parseByQueue(String path, def sectors) {
		File file = new File(path);
		
		int i = 0;
		
		file.withReader { reader ->
			
			ArrayList<Line> linesQueue = null;
			String fullLine;
			int lineNum = 0;
			String line = reader.readLine();
			boolean sectorMatcher = false;
			RuleType currRuleType = null;
			Delimiter currDelim = null;
			
			while (line != null) {
				lineNum++;
				
				
				fullLine = replaceSpecialCharacters(line);;
				line = line.trim();
				
				def matcher = null;
				
				
				for(Delimiter delim: delimQueue) {
					
					sectorMatcher = line ==~ /${delim.getRegex()}/;
					
					if(sectorMatcher) {
						currDelim = delim;
						break;
					}
				}	
				
			
				if(sectorMatcher) {
					
					if(linesQueue != null && !linesQueue.empty) {
						i = addSector(currRuleType,linesQueue, i);
					}
					currRuleType = null;
					
					linesQueue = new ArrayList<Line>();
					Line currLine = new Line(fullLine, null, null, lineNum);
					linesQueue.add(currLine);
					
				} else if(!sectorMatcher) {
					
					if(currDelim != null) {
						for(Rule rule: currDelim.getRules()) {
						
							matcher = line =~ /${rule.getRegex()}/;
							
							if(matcher) {
								currRuleType = rule.getType();
								
								//ZfReport.info("[!FOUND SECTOR!] Line " + lineNum + ": " + currRuleType.getName() + " " + rule.getRegex() + " Full line: " + line);
			
								break;
							}
						
						}
					}
					
					
					if(linesQueue == null) {
						linesQueue = new ArrayList<Line>();
					}
					
					Line currLine = new Line(fullLine, null, null, lineNum);
					linesQueue.add(currLine);
				}
				
				
				line = reader.readLine();
			}
			
			//for last sector
			if(linesQueue != null && !linesQueue.empty) {
				i = addSector(currRuleType,linesQueue, i);
				
			}
			
		}
		
		ZfReport.info("[!FOUND SECTORS!] " + i + " sectors and lines found");
	}
	
	/**
	 * Creates sector and add to sectors object
	 * 
	 * @param currRuleType - rule type
	 * @param linesQueue - list of lines rules
	 * @param i - last founded rule number
	 * 
	 * @return last founded rule number
	 */
	private int addSector(RuleType currRuleType, ArrayList<Line> linesQueue, i) {
		if(currRuleType != null) {
			i++;
			String refToLine = currRuleType.getName().toUpperCase() + i;
			sectors.addSector(linesQueue, currRuleType, refToLine, linesQueue[0].getLineNumber(), linesQueue[linesQueue.size - 1].getLineNumber());
		} else {
			
			
			int index = 0;
			int j = 0;
			
			for(; j<linesQueue.size; j++) {
				
				Line currLine = linesQueue.get(j);
				
				RuleType lineType = null;
				
				for(Rule rule: rulesQueue){
					boolean matcherCurr = currLine.getContent() ==~ /${rule.getRegex()}/;
					
					if(matcherCurr) {
						lineType = rule.getType();
						//ZfReport.info("[!FOUND LINE!] Line " + currLine.getLineNumber() + ": " + lineType.getName() + " " + rule.getRegex() + " Full line: " + currLine.getContent());
						break;
					}
				}
				
				if(lineType != null) {
					if(index < j) {
						sectors.addSector((ArrayList)linesQueue.subList(index, j), linesQueue[index].getLineNumber(), currLine.getLineNumber() - 1);
					}
					i++;
					String refToLine = lineType.getName().toUpperCase() + i;
					ArrayList<Line> currLinesQueue = new ArrayList<Line>();
					currLine.setType(lineType);
					currLinesQueue.add(currLine);
					index = j + 1;
					sectors.addSector(currLinesQueue, lineType, refToLine, currLine.getLineNumber(), currLine.getLineNumber());
					
				}
				
			}
			
			if(index < j) {
				sectors.addSector((ArrayList)linesQueue.subList(index, j), linesQueue[index].getLineNumber(), linesQueue[j - 1].getLineNumber());
			}
			
			
		}
		
		return i;
	}
	
	/**
	 * Creates body of HTML file, that contains references to each founded item
	 *
	 * @param errorSectors - map with founded items
	 *
	 * @return body
	 */
	public String createRefFrame(Map errorSectors) {
		Iterator it = errorSectors.iterator();
		
		String body = "";
		
		while(it.hasNext()) {
			def curr = it.next();
			
			RuleType type = curr.key;
			
			String typeName = type.getName().toUpperCase();
			
			String color = type.getColor();
			
			def currSectors = curr.value;
			
			
			String label = """<a href="javascript:toggleList('${typeName}')" target="_self"><strong>${typeName} (${currSectors.size()})</strong></a><br>""";
			
			String ulLiPart ="""<ul id="${typeName}" style="display: none; margin-left: 0px; padding-left: 3em;">""";
			
			for(int i = 0; i < currSectors.size(); i++) {
				Line line = currSectors.getAt(i).getContent().getAt(0);
				
				String partForLi = line.getContent();
				
				if(partForLi.length() > 50) {
					partForLi = partForLi.substring(0, 50);
				}
				
				ulLiPart +=  """<li>${i+1} <a href="#${currSectors.getAt(i).getRef()}"><span style="color:${color}">[${typeName}] ... ${partForLi} ...</span></a></li><br>"""
			
			}
			
			ulLiPart += "</ul>";
			body += label + ulLiPart;
		}
		
		return body;
	}
	
	/**
	 * Creates body of HTML file, that contains each line from buildLog
	 *
	 * @param sectorsQueue - queue, that contains each sector from buildLog
	 *
	 * @return body
	 */
	public String createContentFrame(def sectorsQueue) {
		StringBuilder body = new StringBuilder("");
		
		int counter = 0;
		
		for(int i = 0; i < sectorsQueue.size(); i++) {
			
			Sector sector = sectorsQueue.getAt(i);
			
			ArrayList<Line> linesQueue = sector.getContent();
			RuleType type = sector.getType();
			
			for(int j = 0; j < linesQueue.size(); j++) {
				
				counter++;
				
				Line line = linesQueue.getAt(j);
				
				if(type == null) {
					body.append(line.getContent() + "\n");
				} else {
					if(j == 0) {
						body.append("""<a name="${sector.getRef()}"></a>""");
					}
					
					body.append("""<span style="color:${type.getColor()}">[${type.getName().toUpperCase()} - ${counter}] ${line.getContent()} </span>\n""");
				}
				
			}
		}
		
		return body.toString();
	}
	
	
	/**
	 * Return all error sectors, which were defined as ErrorFlag
	 * @return
	 */
	public def getParsedMessagesByFlag() {
		if(sectors != null) {
			return sectors.getErrorSectorsByFlag();
		}
		
		return null;
	}
	
	/**
	 * Return error sectors by error type
	 * @param type - error type
	 * @return
	 */
	public def getParsedMessagesByType(String type) {
		if(sectors != null) {
			return sectors.getErrorsByType(type);
		}
		
		return null;
	}
}
