package bootloader

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 *This is the main script of the project. It used to run build.
 *At the end the script copy results files to the share.
 */
 
import groovy.text.Template
import project_lib.LicenseWorker

import java.io.BufferedReader
import java.io.BufferedWriter
import java.io.File
import java.io.FileNotFoundException
import java.io.FileReader
import java.io.FileWriter
import java.io.IOException
import java.io.InputStreamReader
import java.nio.file.Path
import java.nio.file.Paths
import java.text.SimpleDateFormat
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter

import project_lib.ProjectUtils
import lib_groovy.*
import project_lib.CMTChecker
import project_lib.VariantConfig 
import groovy.io.FileType




/////////////////////////////////////////////////SCRIPT VARIABLES/////////////////////////////////////////////

//Map with the script configuration
Map	configMap
//Disk letter of the test view
String resultLetter
//Developer of the builded version
String developer
//Type from script parameter (debug, release)
String type

String viewName

/////////////////////////////////////////////////////RUN//////////////////////////////////////////////////////

LicenseWorker lWorker = null
boolean isCANape = false

try {

	//AURIX version
	String aurixV = System.getenv("AURIXV")


	//Loads build type from script parameter (debug, release)
	type = args[0]
	
	//Load configuration
	String pathToConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + "Test.xml").toString()
	configMap = ProjectUtils.loadAndValidateConfig(pathToConfig, type)

	Path resultDirPath = Paths.get(ProjectUtils.foundCreatedDir(configMap["resultDir"]))
	resultDirPath = resultDirPath.resolve(type)


	//Creates and starts result view
	viewName = configMap["viewName"].replace("%COMPUTERNAME%", System.getenv("COMPUTERNAME")).replace("%TYPE%", type)

	ClearCase.createView(viewName, false, resultDirPath.resolve(configMap["configSpecTime"]).toString())

	//Mount required vobs
	ClearCase.mountMultipleVobs(configMap["requiredVobs"])


	//Check if result view is mounted to drive letter and mounts it if its not.
	try{
		resultLetter = Utils.mountViewToLetter(viewName, false)
	} catch(Exception e) {
		System.exit(5)
	}

	Path pathToDrive = Paths.get(resultLetter)
	
	//Prepares path variables for the build output and result directory .
	Path outputDirPath = pathToDrive.resolve(configMap["outputDir"])

	//Creates buildLogs.txt for storing logs of the build.
	Path logPath = resultDirPath.resolve("test").resolve("buildLogs.txt")
	File logsFile = new File(logPath.toString())
	logsFile.getParentFile().mkdirs()

	String[] components = configMap["components"].split(",")

	String pathToBatch
	String batchName
	String script
	Map procResult
	String errorLevelOk = "Error level is 0"

	for(String component: components){
		Map	compConfig = ProjectUtils.loadAndValidateConfig(pathToConfig, component.trim())
		//Prepares variables for calling build batch
		if(compConfig == null){
			continue
		}

		//Run batches
		runScript(resultLetter, compConfig["scriptGenerator"], logsFile)

		runScript(resultLetter, compConfig["certGenerator"], logsFile)

		runScript(resultLetter, compConfig["mdtGenerator"], logsFile)

		runScript(resultLetter, compConfig["buildAndTest"], logsFile)
		
		int sleepTime = 300000

        sleep(sleepTime)

        boolean isTested = false

        int n = 3

        while(n > 0 || !isTested){
            isTested = checkOutput(compConfig, resultLetter)
            n--
            sleep(sleepTime)
        }

		runScript(resultLetter, compConfig["stopBatch"] as String, logsFile)

		runScript(resultLetter, compConfig["checker"] as String, logsFile)

		copyFiles(compConfig, configMap, pathToDrive, resultDirPath)
	}


    Utils.unmountDiskLetter(resultLetter, false)
    ClearCase.removeView(viewName)

} catch ( Exception e ) {
	ZfReport.fatal(e.message)
	ZfReport.fatal(e.stackTrace)

	if (configMap != null) {
		ProjectUtils.sendMailFatal(configMap["EmailRecipientsFatal"])

		try{
			Utils.unmountDiskLetter(resultLetter, false)
			if(type == "release") {
				ClearCase.removeView(viewName)
			}
		} catch ( Exception ex ) {
			ZfReport.fatal(ex.message)
			ZfReport.fatal(ex.stackTrace)
		}
	}

	System.exit(1)
}


private void runScript(String resultLetter, String batch, File logsFile){
	batch = "${resultLetter}${batch}"

	int index = batch.lastIndexOf("\\")
	String pathToBatch = batch.substring(0, index)
	String batchName = batch.substring(index + 1, batch.length())

	//Prepare script for running.
	script = "cmd /c cd /D ${pathToBatch} & call ${batchName} -a"
	//Run script.
	Map procResult = WinUtils.execute(script)
	logsFile << procResult["text"].toString() + "\n"
}


/**
 * Creates batches file objects
 *
 * @param batches - contains paths to batches
 * @param resultLetter - letter on that was view mounted
 * @param componentType - ZF Boot
 *
 * @return file objects
 */
private static def openBatches(String batches, String resultLetter) {
	String[] pathes = batches.split(",")

	String[] files = new String[pathes.length]

	for(int i=0 i<pathes.length i++) {
		files[i] = "${resultLetter}${pathes[i]}"
	}

	return files
}

/**
 * Copies files from View to the share and creates CMT checker
 *
 * @param outputDir - path to the output directory
 * @param componentType - component type
 * @param resultDirPath - path to the share
 * @param configMap - Map with the script configuration
 * @param resultLetter - drive letter
 */
private static void copyFiles(Map compConfig, Map configMap, Path pathToDrive, Path resultDirPath){

	String[] outputFilesNames = compConfig["outputFilesCopy"].split(",")


	//Path to output build directory
	Path outputPath = pathToDrive.resolve(configMap["outputDir"] + "")
	//Path to directory, in which will be copy results of build
	Path resultPath = resultDirPath.resolve(configMap["componentType"] + "").resolve(compConfig["outputPostfix"] + "")

	//Iterates through files
	for(int k=0 k<outputFilesNames.length k++) {


		String pathToCurrFile = outputPath.resolve(outputFilesNames[k]).toString()
		File currFile = new File(pathToCurrFile)

		//If current file does not exist, then new iteration
		if(!currFile.exists()) {
			ZfReport.info("File ${pathToCurrFile} does not exist.")
			continue
		}

		//Copies current file from output build folder to the result directory (from View to share)
		Utils.copy(outputPath.toString(), resultPath.toString(), outputFilesNames[k])


	}


}

/**
 * Check if all test were passed
 * @param configMap - configuration map
 * @param resultLetter - drive letter
 * @return
 */
private boolean checkOutput(def configMap, String resultLetter) {
    String[] outputFiles = configMap["outputLogs"].split(",")

    for(String fileName: outputFiles){
        String currPath = resultLetter + Paths.get(configMap["outputLogDir"]).resolve("fileName")

        File currFile = new File(currPath)

        if(!currFile.exists()) {
            return false
        }
    }

    return true
}