package project_lib;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITES7
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// This library reads the CMT *.xls files and allows read, parse and compare it by vG, MI and C2CR fields.
// Also this library allows create HTML with results.
//------------------------------------------------------------------------------


class CMTChecker {

	private static String isOk;
	/**
	 * Parses CMT *.xls file to map
	 * @param path - path to CMT file
	 * @return Map in the form of [line = Map[name = value]]
	 * 
	 * @throws Exception
	 */
	public static def parse(String path) throws Exception {
		isOk = "passed";
		//checks if file exist and does not directory
		checkFile(path);
		
		BufferedReader bReader = new BufferedReader(new FileReader(path));
		
		String line;
		
		def cmtMap = [:];
		
		//Array for column names
		List titles = [];
		
		//Reads *.xls file line by line
		for(int i = 0;(line = bReader.readLine()) != null; i++) {
			
			String[] currLine = line.split("	");
			
			//Fills column names array
			if(i == 0) {
				titles.addAll(currLine);
				continue;
			}
			
			//Creates Map for current row
			def currMap = [:];
			
			//Creates Map from row
			for(int j = 0; j < titles.size(); j++) {
				//deletes " symbols from start and end of item
				String normalize = ( ( currLine[j].charAt(0) == "\"" ) && ( currLine[j].charAt(currLine[j].length() - 1) == "\"" ) ) 
									? currLine[j].substring(1, currLine[j].length() - 1) 
									: currLine[j];
				currMap.put(titles[j], normalize);
			}
			
			cmtMap.put(currMap.get("Measured object"), currMap);
		}
		
		return cmtMap;
	}
	
	/**
	 * Checks if file exist and does not directory
	 * @param path - path to the file
	 * @throws Exception
	 */
	static void checkFile(String path) throws Exception{
		File file = new File(path);
		
		if(!file.exists() || !file.isFile()) {
			throw new Exception("Error: file does not exist or file is directory!");
		}
	}
	
	/**
	 * Creates New all tracked part
	 * 
	 * @param cmtMapNew - CMT Map, that contains content of new CMT *.xls file
	 * @param configMap - map with min/max values
	 * @param pathToTmpl - path to table template
	 * 
	 * @return html table
	 */
	public static String countMetrixes(def cmtMapNew, def configMap, String pathToTmpl) {
		String tableBody = (new File(pathToTmpl)).text;
			
		Iterator it = cmtMapNew.iterator();
		
		boolean isFunctionsConf = configMap["functions"].empty ? false : true;
		
		while(it.hasNext()) {
			String cssClass = "";
			String row = "<tr>";
			
			def currFunction = it.next();
			
			row += "<th class='function'>${currFunction.key}</th>\n";
			
			Map currConfig = isFunctionsConf ? getCurrentConfig(configMap, currFunction.key) : configMap["global"];
			
			
			/**v(G) - Cyclomatix complexity*/
			int vGMin = Integer.parseInt(currConfig["vGMin"]);
			int vG = Integer.parseInt(currFunction.value["v(G)"]);
			int vGMax = Integer.parseInt(currConfig["vGMax"]);

			if((vGMin > vG) || (vG > vGMax)){
				cssClass = "wrong-value";
				isOk = "failed";
			} else {
				cssClass = "normal-value";
			}
			
			row += "<th>${vGMin}</th>\n";
			row += "<th class='${cssClass}'>${currFunction.value["v(G)"]}</th>\n";
			row += "<th>${vGMax}</th>\n";
			
			/**MI - Maintainability Index*/
			int MIMin = Integer.parseInt(currConfig["MIMin"]);
			int MI = Integer.parseInt(currFunction.value["MI"]);
			int MIMax = Integer.parseInt(currConfig["MIMax"]);

			if((MIMin > MI) || (MI > MIMax)){
				cssClass = "wrong-value";
				isOk = "failed";
			} else {
				cssClass = "normal-value";
			}
			
			row += "<th>${MIMin}</th>\n";
			row += "<th class='${cssClass}'>${MI}</th>\n";
			row += "<th>${MIMax}</th>\n";
			
			/**C2CR - Comment to Code Ratio*/
			
			double C2CR = 100 * (double)Integer.parseInt(currFunction.value["LOCcom"]) / (double)Integer.parseInt(currFunction.value["LOCpro"]);
			
			int C2CRMin = Integer.parseInt(currConfig["C2CRMin"]);
			int C2CRMax = Integer.parseInt(currConfig["C2CRMax"]);

			if((C2CRMin > C2CR) || (C2CR > C2CRMax)){
				cssClass = "wrong-value";
				isOk = "failed";
			} else {
				cssClass = "normal-value";
			}
			
			row += "<th>${C2CRMin}</th>\n";
			row += "<th class='${cssClass}' title='${C2CR}'>${Math.round(C2CR)}</th>\n";
			row += "<th>${C2CRMax}</th>\n";
			
			
			row += "</tr>\n";
			
			tableBody += row;
		}
		
		tableBody += "</table>\n</div>\n";
		
		return tableBody;
	}
	
	/**
	 * Write file to share
	 * 
	 * @param path - path to file
	 * @param html - content of file
	 */
	static void writeToHtml(String path, String html){
		new File("${path}").text = html;
	}
	
	/**
	 * Creates final html file
	 * 
	 * @param newAllTracked - New all tracked part
	 * @param changesTracked - Changes tracked part
	 * @param oldAll - Old all part
	 * @param newAll - New all part
	 * @param pathToTmpl - path to template
	 * 
	 * @return html
	 */
	public static String createMainHtml(String newAllTracked, String changesTracked, String oldAll, String newAll, String pathToTmpl) {
		
		String cmtTemplate = (new File(pathToTmpl)).text;
		
		int pos = cmtTemplate.indexOf("</body>");
		
		String resultHtml = cmtTemplate.substring(0, pos-1);
		
		resultHtml += newAllTracked + changesTracked + oldAll + newAll;
		
		resultHtml += cmtTemplate.substring(pos);
		
		return resultHtml;
	}
	
	/**
	 * Creates Changes tracked part
	 *
	 * @param cmtMapNew - CMT Map, that contains content of new CMT *.xls file
	 * @param cmtMapOld - CMT Map, that contains content of old CMT *.xls file
	 * @param configMap - map with min/max values
	 * @param pathToTmpl - path to table template
	 *
	 * @return html table
	 */
	public static String compareNewAndOld(def cmtMapNew, def cmtMapOld, def configMap, String pathToTmpl) {
		String tableBody = (new File(pathToTmpl)).text;
		
		Iterator it = cmtMapNew.iterator();
		
		boolean isFunctionsConf = configMap["functions"].empty ? false : true;
		
		while(it.hasNext()) {
			boolean correction = false;
			def currentFunction = it.next();
			String functionName = currentFunction.key;
			def oldFunction = cmtMapOld.getAt(functionName);
			
			int vGOld = 0;
			int MIOld = 0;
			double C2CROld = 0.0;
			
			if(oldFunction != null) {
				vGOld = Integer.parseInt(oldFunction["v(G)"]);
				MIOld = Integer.parseInt(oldFunction["MI"]);
				C2CROld = 100 * (double)Integer.parseInt(oldFunction["LOCcom"]) / (double)Integer.parseInt(oldFunction["LOCpro"]);
			}
			
			int vGNew = Integer.parseInt(currentFunction.value["v(G)"]);
			int MINew = Integer.parseInt(currentFunction.value["MI"]);
			double C2CRNew = 100 * (double)Integer.parseInt(currentFunction.value["LOCcom"]) / (double)Integer.parseInt(currentFunction.value["LOCpro"]);
			
			if(vGNew == vGOld && MINew == MIOld) { continue; }
			
			String cssClass = "";
			String row = "<tr>";
			
			row += "<th>${functionName}</th>\n";
			
			Map currConfig = isFunctionsConf ? getCurrentConfig(configMap, functionName) : configMap["global"];
			
			/**v(G) - Cyclomatix complexity*/
			int vGMin = Integer.parseInt(currConfig["vGMin"]);
			int vGMax = Integer.parseInt(currConfig["vGMax"]);
			int vGDiff = vGNew - vGOld;
			
			boolean isWrongVG = ((vGMin > vGNew) || (vGNew > vGMax)) ? true : false;

			if(isWrongVG){
				cssClass = "wrong-value";
				isOk = "failed";
				correction = true;
			} else {
				cssClass = "normal-value";
			}

			
			row += "<th>${vGMin}</th>\n";
			row += "<th class='${cssClass}'>${vGNew}</th>\n";
			row += "<th>${vGMax}</th>\n";
			row += "<th>${vGDiff}</th>\n";
			
			/**MI - Maintainability Index*/
			int MIMin = Integer.parseInt(currConfig["MIMin"]);
			int MIMax = Integer.parseInt(currConfig["MIMax"]);
			int MIDiff = MINew - MIOld;
			
			boolean isWrongMI = ((MIMin > MINew) || (MINew > MIMax)) ? true : false;

			if(isWrongMI){
				cssClass = "wrong-value";
				isOk = "failed";
				correction = true;
			} else {
				cssClass = "normal-value";
			}
			
			row += "<th>${MIMin}</th>\n";
			row += "<th class='${cssClass}'>${MINew}</th>\n";
			row += "<th>${MIMax}</th>\n";
			row += "<th>${MIDiff}</th>\n";
			
			/**C2CR - Comment to Code Ratio*/
			
			int C2CRMin = Integer.parseInt(currConfig["C2CRMin"]);
			int C2CRMax = Integer.parseInt(currConfig["C2CRMax"]);
			double C2CRDiff = C2CRNew - C2CROld;
			
			boolean isWrongC2CR = ((C2CRMin > C2CRNew) || (C2CRNew > C2CRMax)) ? true : false;

			if(isWrongC2CR){
				cssClass = "wrong-value";
				isOk = "failed";
				correction = true;
			} else {
				cssClass = "normal-value";
			}
			
			row += "<th>${C2CRMin}</th>\n";
			row += "<th class='${cssClass}' title='${C2CRNew}'>${Math.round(C2CRNew)}</th>\n";
			row += "<th>${C2CRMax}</th>\n";
			row += "<th title='${C2CRDiff}'>${Math.round(C2CRDiff)}</th>\n";
			
			row += (correction) ? "<th>Yes</th>\n" :  "<th>No</th>\n";
			
			row += "<th></th>\n";
			
			row += "</tr>\n";
			
			tableBody += row;
		}
		
		tableBody += "</table>\n</div>\n";
		
		return tableBody;
	}
	
	/**
	 * Create html table from map
	 * 
	 * @param map - map creates from *.xls
	 * @param pathToTmpl - path to html templates
	 * @return html table
	 */
	public static String allMapToHtmlTable(def map, String pathToTmpl) {
		String tableBody = "";
		
		Iterator it = map.iterator();
		Iterator itRow;
		
		while(it.hasNext()) {
			def currFunction = it.next();
			
			if(tableBody.empty) {
				tableBody += "<tr>";
				
				itRow = currFunction.value.iterator();
				
				while(itRow.hasNext()) {
					def currElement = itRow.next();
					
					tableBody += "<th>${currElement.key}</th>\n";
				}
				
				tableBody += "</tr>\n";
			}
			
			tableBody += "<tr>";
			
			itRow = currFunction.value.iterator();
			
			while(itRow.hasNext()) {
				def currElement = itRow.next();
				
				tableBody += "<th>${currElement.value}</th>\n";
			}
			
			tableBody += "</tr>\n";
		}
		
		tableBody = (new File(pathToTmpl)).text + tableBody; 
		
		tableBody += "</table>\n</div>\n";
		
		return tableBody;
	}
	
	/**
	 * Returns global config Map, if does not exist configuration Map for current function
	 * @param configMap - Map of parameters [global = Map[name=value], functions = Map[name=value]]
	 * @param functionName - name of current function
	 * @return Map [name=value]
	 */
	private static Map getCurrentConfig(Map configMap, String functionName) {
		if(configMap["functions"].containsKey(functionName)) {
			return configMap["functions"][functionName];
		}
		return configMap["global"];
	}

	static String isOk(){
		return isOk;
	}

}