package project_lib

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITES7
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------------
// This class contains methods for different cases
//------------------------------------------------------------------------------

import java.io.File
import java.util.Map

import lib_groovy.ZfReport
import lib_groovy.Config
import lib_groovy.EmailUtil
import java.text.DateFormat
import java.text.SimpleDateFormat
import java.util.zip.ZipEntry
import java.util.zip.ZipOutputStream

class ProjectUtils{

	///////////////////////////////////////////////////METHODS////////////////////////////////////////////////////


	/**
	 * Loads and validates configuration from config.xml
	 * @return map with the script configuration
	 * @throws Exception when file config.xml is not found or wrong value is found within.
	 */
	public static Map loadAndValidateConfig(String path, String type) throws Exception {

		def map = Config.readConfigFile(path, type)

		ZfReport.info("Config.xml has been loaded and validated.")

		return map
	}


	/**
	 * Sends email with information about build result in case of fatal error.
	 * 
	 * @param EmailRecipients - recipients mails. For example, "mail1,mail2,mail3"
	 * @return
	 */
	public static sendMailFatal(String EmailRecipients) {

		String emailSubject = "\"FATAL FAILURE IN ${System.getenv('JOB_NAME')} BUILD\""
		String emailBody = "FATAL FAILURE occured in ${System.getenv('JOB_NAME')} BUILD"

		sendMail(EmailRecipients, emailSubject,  emailBody)
	}

	/**
	 * Sends email with information about build result.
	 * 
	 * @param EmailRecipients - recipients mails. For example, "mail1,mail2,mail3"
	 * @param emailSubject
	 * @param emailBody
	 */
	public static sendMail(String EmailRecipients, String emailSubject, String emailBody) {
		String emailFrom = "email"
		EmailUtil.sendEmail(emailFrom, EmailRecipients, emailSubject, emailBody)
	}
	
	/**
	 * Founds last created folder inside directory
	 * 
	 * @param resultDirPath - path to directory
	 * 
	 * @return founded folder
	 */
	public static String foundCreatedDir(String resultDirPath) {
		try{
			List listOfDir = []
			File dir = new File(resultDirPath)
			
			dir.eachDir{ file -> listOfDir << file}
			
			if(listOfDir.size() < 1) { 
				throw new Exception("Directory does not exist!") 
			}
			
			listOfDir.sort{it.lastModified()}
			listOfDir.reverse(true)
			
			return listOfDir[0].getAbsolutePath()
		} catch ( Exception e ) {
			ZfReport.fatal(e)
			System.exit(1)
		}
	}
	
	/**
	 * Cut main part from ConfigSpec and
	 * added time conditions to it
	 *
	 * @param configSpecTmpl - path to ConfigSpec
	 * @param csTime - time for replacement
	 * @param ccVersion - ConfigSpec version
	 * @return modified ConfigSpec
	 */
	public static String createConfigSpec(String configSpecTmpl, String csTime, Integer ccVersion = -1) {
		
		ccVersion = ccVersion != null ? ccVersion : -1
		
		String regex = "^([^#](.{1,}))([/LATEST]|[\\\\LATEST])(.{0,})"
		String configSpec = ""
		
		new File(configSpecTmpl).withReader { reader ->
			def line
			while((line = reader.readLine()) != null) {
				if(line.contains("END OF CONFIGSPEC")) {
					break
				}
				
				line = line.trim()
				
				def matcher = line =~ /${regex}/
				
				if(matcher) {
					line = line.replace("LATEST", "LATEST -time " + csTime)
				}
				
				configSpec += line + "\r\n"
			}
		}
		
		if(ccVersion != -1) {
			configSpec += "#ConfigSpec version is " + ccVersion + "\r\n"
			configSpec += "##############################################\r\n"
		}
		
		
		return configSpec
	}
	
	public static String getTimeStamp() {
		DateFormat dateFormat = new SimpleDateFormat("dd.MM.yy-HH:mm:ss")
		Date date = new Date()
		return dateFormat.format(date)
	}

	/**
	 * This method zips the directory
	 * @param dirPath - path to directory that will zipped
	 * @param zipDirName - path to future zip file
	 */
	static void zipDirectory(String dirPath, String zipDirName) {
		try {
			File dir = new File(dirPath)
			List<String> filesListInDir = new ArrayList<String>()
			populateFilesList(dir, filesListInDir)
			//now zip files one by one
			//create ZipOutputStream to write to the zip file
			FileOutputStream fos = new FileOutputStream(zipDirName)
			ZipOutputStream zos = new ZipOutputStream(fos)
			for(String filePath : filesListInDir){
				//for ZipEntry we need to keep only relative file path, so we used substring on absolute path
				ZipEntry ze = new ZipEntry(filePath.substring(dir.getAbsolutePath().length()+1, filePath.length()))
				zos.putNextEntry(ze)
				//read the file and write to ZipOutputStream
				FileInputStream fis = new FileInputStream(filePath)
				byte[] buffer = new byte[1024]
				int len
				while ((len = fis.read(buffer)) > 0) {
					zos.write(buffer, 0, len)
				}
				zos.closeEntry()
				fis.close()
			}
			zos.close()
			fos.close()
		} catch (IOException e) {
			e.printStackTrace()
		}
	}

	/**
	 * This method allows to delete a file or folder.
	 * @param path - path to directory or file
	 * @return true is deleted, false is not deleted
	 */
	static boolean deleteFileOrDir(String path){
		File file = new File(path)

		if(!file.exists()) {
			ZfReport.warning("File " + path + " does not exist")
			return false
		}

		if(file.isFile()){
			try {
				file.delete()
				ZfReport.info("File " + path + " was deleted")
				return true
			} catch (Exception e){
				ZfReport.warning("Some problem deleting the file " + path)
				return false
			}
		}

		if(file.isDirectory()){
			try {
				boolean isDeleted = file.deleteDir()
				if(isDeleted){
					ZfReport.info("Directory " + path + " was deleted")
					return true
				} else {
					ZfReport.warning("Some problem deleting the directory " + path)
					return false
				}
			} catch (Exception e){
				ZfReport.warning("Some problem deleting the directory " + path)
				return false
			}
		}


	}

	/**
	 * This method populates all the files in a directory to a List
	 * @param dir - directory that will zipped
	 * @param filesListInDir - list of files, which will zipped
	 * @throws IOException
	 */
	private static void populateFilesList(File dir, List<String> filesListInDir) throws IOException {
		File[] files = dir.listFiles()
		for(File file : files){
			if(file.isFile()) filesListInDir.add(file.getAbsolutePath())
			else populateFilesList(file, filesListInDir)
		}
	}


}