package bootloader

import java.nio.file.Path
import java.nio.file.Paths

import java.util.concurrent.atomic.AtomicInteger

import project_lib.ProjectUtils
import project_lib.LintCompare
import lib_groovy.*

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**
 * This script runs in a last stage of test Jenkins job. It sends email to the persons
 * specified in config and to the developer of the tested version.
 */


/////////////////////////////////////////////////SCRIPT VARIABLES/////////////////////////////////////////////

//Map with script configuration
Map	configMap
//Exit code (0 - ok, 1 - warning, 2 - error)
AtomicInteger exitCode = new AtomicInteger(0)



/////////////////////////////////////////////////////RUN//////////////////////////////////////////////////////

try {
	//Aurix version
	String aurixV = System.getenv("AURIXV")
	//Loads build type from script parameter (debug, release)
	String type = args[0]

	String pathToConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + "Test.xml").toString()

	configMap = ProjectUtils.loadAndValidateConfig(pathToConfig, type)

	Path resultDirPath = Paths.get(ProjectUtils.foundCreatedDir(configMap["resultDir"]))

	String mailTemplatePath = Paths.get(System.getenv("SOURCEPATH")).resolve("tmpl\\template.html").toString()

	String mailTemplateFile = (new File(mailTemplatePath)).text

	int pos = mailTemplateFile.indexOf("</body>")

	String resultHtml = mailTemplateFile.substring(0, pos-1)

	resultHtml += createMailBody(resultDirPath.resolve(type), aurixV, configMap, exitCode, type)

	resultHtml += mailTemplateFile.substring(pos)

	String emails = configMap["EmailRecipientsSuccess"]

	String pathToHtmlMail = resultDirPath.resolve(type).resolve("mail.html").toString()
	
	//Sends email
	sendMailBuild((configMap['projectName']).toUpperCase(), emails, resultHtml, "", configMap['ResponsibleName'], configMap['ResponsibleEmail'], exitCode.value, pathToHtmlMail)
	ZfReport.info("Mail sent.")
	
	return

} catch ( Exception e ) {
	ZfReport.fatal(e.getMessage())
	ZfReport.fatal(e.getStackTrace())
	
	if (!configMap.equals(null)) {
		ProjectUtils.sendMailFatal(configMap["EmailRecipientsFatal"])
	}
	System.exit(1)
}

/**
 * Sends email with information about build result.
 *
 * @param projectName - project name
 * @param EmailRecipients - recipients mails. For example, "mail1,mail2,mail3"
 * @param build - build name
 * @param time - build start time
 * @param responsibleName - responsible person's name
 * @param responsibleEmail - responsible person's email
 * @param exitCode - exit code (0 - ok, 1 - warning, 2 - error)
 */
public static sendMailBuild(String projectName, String EmailRecipients, String build, String time, String responsibleName, String responsibleEmail, int exitCode, String pathToHtmlMail) {

	String emailSubject
	String emailBody

	String stateOfBuild

	switch(exitCode) {
		case 0:
			stateOfBuild = "OK"
			break
		case 1:
			stateOfBuild = "WARNING"
			break
		case 2:
		case 3:
			stateOfBuild = "FAILURE"
			break
	}

	emailSubject = "\"The result of build ${projectName} ${time} is " + stateOfBuild + "\""

	emailBody = """
					<table>
							<tr>
								  <td>  Hello,<br>
								   this email contains informations about ${projectName}<strong> ${time} </strong></td><br>
								   
							</tr>
							<tr>
								  <td> ${build} </td>
							</tr>

							<tr style="border: none">
								   <td style="border: none"> 
									<br><br>_______
									<br>This mail was generated by a Continuous Integration job.
									<br>In case of any problem please contact the job responsible: <a href="mailto:${responsibleEmail}">${responsibleName}</a>
									<br>In case the responsible is not available, send e-mail to: <a href="mailto:PLZ-Tooling-CI-Support">PLZ-Tooling-CI-Support</a>.</td>
							</tr>
					</table>"""



	File mailHtml = new File(pathToHtmlMail)

	mailHtml.createNewFile()

	mailHtml.write(emailBody)

	ZfReport.info("Mail saved to " + pathToHtmlMail + ".")

	ProjectUtils.sendMail(EmailRecipients, emailSubject, emailBody)
}


/**
 * Creates email body
 *
 * @param resultDirPath - path to result directory
 * @param aurixV - aurix version
 * @param configMap - map with script configuration
 * @param exitCode - exit code
 * @param type - debug or realise
 *
 * @return email body
 */
String createMailBody(Path resultDirPath, String aurixV, def configMap, AtomicInteger exitCode, String type) {

	//Creates component type array
	String[] componentTypeArr = configMap["componentType"].split(",")
	String mailBody = ""
	Map<String, Boolean> compComb = [:]
	Map resultMap = [:]

	for(String componentType: componentTypeArr){
		String pathToConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + componentType.toUpperCase() + ".xml").toString()
		def currConfMap = ProjectUtils.loadAndValidateConfig(pathToConfig, type)
		String[] mailComps = currConfMap["mailComponent"].split(",")

		for(String mailComp: mailComps){
			resultMap.put(mailComp, [:])
		}

		Path pathToComponent = resultDirPath.resolve(componentType)


		String pathToBuildLog = pathToComponent.resolve("buildLogs.txt").toString()
		String pathToBuildLogHTML = pathToComponent.resolve("buildLogs.html").toString()
		String pathToRules = Paths.get(System.getenv("SOURCEPATH")).resolve(configMap["rulesFile"]).toString()

		boolean isOk = false
		try{
			isOk = checkBuildLog(pathToBuildLog, pathToRules, pathToBuildLogHTML)
		} catch (Exception e) { }

		String[] components = currConfMap["components"].split(",")

		for(String component: components){
			component = component.trim()
			String currMailComp = ""

			for(String mailComp: mailComps){
				if(component.contains(mailComp)){
					currMailComp = mailComp
				}
			}

			Map compMap = ProjectUtils.loadAndValidateConfig(pathToConfig, component)

			Map componentResult = [:]

            def currMailCompMap = resultMap.get(currMailComp)

            if(!currMailCompMap.containsKey("componentDefs")){
                Map componentDefs = [:]

                currMailCompMap.put("componentDefs", componentDefs)
                componentDefs.put("mailComp", currMailComp)
                componentDefs.put("pathToBuildLog", pathToBuildLog)
                componentDefs.put("pathToBuildLogHTML", pathToBuildLogHTML)
                componentDefs.put("resultDir", pathToComponent)
                componentDefs.put("buildStatus", "ok")

            }

            currMailCompMap.put(component, componentResult)

			checkFiles(pathToComponent, componentResult, compMap)
            String buildStatus = componentResult.get("buildStatus")

            if(buildStatus.equals("failure")){
                currMailCompMap.get("componentDefs").put("buildStatus", "failure")
                exitCode.set(2)
            } else if(buildStatus.equals("warning") && !currMailCompMap.get("componentDefs").get("buildStatus").equals("failure")) {
                currMailCompMap.get("componentDefs").put("buildStatus", "warning")
                if(exitCode.value != 2){
                    exitCode.set(1)
                }
            }



		}

	}


	resultMap.each{ key, val ->
		mailBody += createResultTable(key, val)
	}



	return mailBody
}

/**
 * Checks are files exist and ok
 *
 * @param resultDirPath - path to the result folder
 * @param componentResult - map for current component
 * @param compMap - data of component
 * @param component - component name
 */
void checkFiles(Path resultDirPath, def componentResult,  def compMap) {

	componentResult.put("compComb", "ok")
	componentResult.put("buildStatus", "ok")
    componentResult.put("variant", compMap.get("variant") + "")

	String notFoundFiles = ""
	//0 - build ok, 1 - build warning, 2 - build error in compiler_combined.err, 3 - *.hex or *.s19 is not exist

	String postfix = compMap["outputPostfix"]

	Path pathToCurrDirr = resultDirPath.resolve(postfix)

	String[] outputFilesArr = compMap["outputFilesCopy"].split(",")

	for(int j=0; j<outputFilesArr.length; j++) {
		String pathToCurrFile = pathToCurrDirr.resolve(outputFilesArr[j]).toString()

		File currFile = new File(pathToCurrFile)
		if(!currFile.exists()) {
			notFoundFiles += outputFilesArr[j] + ", "
			if(componentResult.get("buildStatus").equals("ok")) {
				componentResult.put("buildStatus", "warning")
			}
		}
	}

	if(!notFoundFiles.empty){
		notFoundFiles = notFoundFiles.substring(0, notFoundFiles.length() - 2)
	}

	componentResult.put("notFoundFiles", notFoundFiles)
}

/**
 * Creates build result table
 *
 * @param mailComponent - name of the component
 * @param resultMap - result data
 */
public String createResultTable(String mailComponent, def resultMap) {

	String notExistBlock = ""


	resultMap.each { key, val ->
        if(!key.equals("componentDefs")){
			if(!val.get("notFoundFiles").empty){
				String variant = val.get("variant")
				String notFoundFiles = val.get("notFoundFiles")
				notExistBlock += "<br><span><strong>WARNING in variant ${variant}: </strong>files ${notFoundFiles} were not created!</span><br>"
			}
        }


	}

    String table = notExistBlock


    String body = getbuildBodyHeader(resultMap.get("componentDefs"), table)

	return body
}


/**
 * Checks build log by build log parser
 *
 * @param pathToBuildLog - path to the BuildLog
 * @param pathToRules - path to the rules file
 * @param pathToOutputFile - path to the output file
 * @param pathToXMLConfig - path to the xml configuration file
 *
 * @return true or false
 */
boolean checkBuildLog(String pathToBuildLog, String pathToRules, String pathToOutputFile, String pathToXMLConfig = null) {
	BuildLogParser parser = new BuildLogParser()

	boolean isErr = (pathToXMLConfig == null) ? parser.run(pathToRules, pathToBuildLog, pathToOutputFile) : parser.run(pathToRules, pathToBuildLog, pathToOutputFile, pathToXMLConfig)

	return isErr
}

/**
 * Creates header before table with information about status of build
 *
 * @param results - map with main results of the build
 * @param buildBody - html body
 *
 * @return header before table
 */
String getbuildBodyHeader(Map results, String buildBody) {
    String body = ""

    if(results.get("buildStatus").equals("ok")) {
        body = """
							<br>Status of build <strong>${results.get("mailComp")}</strong> is: <strong><font color=\"green\">OK</font></strong><br> See the results <a href=\"${results.get("resultDir")}\">here</a> 
							<br> See the logs <a href="${results.get("pathToBuildLog")}">here</a>
							<br> See the logs in HTML format <a href="${results.get("pathToBuildLogHTML")}">here</a> 
							<br>
							${buildBody}
							<br>
						"""
    } else if(results.get("buildStatus").equals("warning")) {
        body = """
							<br>Status of build <strong>${results.get("mailComp")}</strong> is: <strong><font color=\"yellow\">WARNING</font></strong><br> See the results <a href=\"${results.get("resultDir")}\">here</a>
							<br> See the logs <a href="${results.get("pathToBuildLog")}">here</a>  
							<br> See the logs in HTML format <a href="${results.get("pathToBuildLogHTML")}">here</a>
							<br>
							${buildBody}
							<br>
						"""
    } else if(results.get("buildStatus").equals("failure")) {
        body = """
							<br>Status of build <strong>${results.get("mailComp")}</strong> is: <strong><font color=\"red\">FAILURE</font></strong><br> See the results <a href=\"${results.get("resultDir")}\">here</a> 
							<br> See the logs <a href="${results.get("pathToBuildLog")}">here</a>
							<br> See the logs in HTML format <a href="${results.get("pathToBuildLogHTML")}">here</a>
							<br>
							${buildBody}
							<br>
						"""
    }

    return body
}
