//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//	Company    : ZF Engineering, Pilsen, Czech Republic
//  Department : DITIS
//  Project    : bootloader Jenkins
//  Developer  : Ihar Auchynnikau <ihar.auchynnikau@zf.com>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


/**
 * This script is used to determine whether a new version has been issued for building, and if so,
 * it create the  lastJenkins.xml with information about the build for the next stages.
 * If no new version is available job ends and it removed from Jenkins build history.
 */

package bootloader;

import project_lib.ProjectUtils
import lib_groovy.*

import java.nio.file.Paths
import java.text.SimpleDateFormat

/////////////////////////////////////////////////SCRIPT VARIABLES/////////////////////////////////////////////

//Map with the configuration of the script
Map	configMap
//Info about last build
XmlStorage myXmlStorage
//CC version of the build
int ccVersion
//Last builded version
int	lastVersion
//View name
String viewNameCS

/////////////////////////////////////////////////////RUN//////////////////////////////////////////////////////

try {

	//AURIX version
	String aurixV = System.getenv("AURIXV")
	//Loads build type from script parameter (debug, release)
	String type = args[0]

	//Load configuration
	String pathToConfig = Paths.get(System.getenv("scriptFolder")).resolve("config\\bootloaderConfig" + aurixV.toUpperCase() + ".xml").toString()

	configMap = ProjectUtils.loadAndValidateConfig(pathToConfig, type)

	
	//Creates and starts view
	viewNameCS = configMap["viewName"].replace("%COMPUTERNAME%", System.getenv("COMPUTERNAME")).replace("%TYPE%", type) + "_CHECK"
	
	String pathToCS = Paths.get(System.getenv("SOURCEPATH")).resolve(configMap["configSpecTmpl"]).toString()
	
	ClearCase.createView(viewNameCS, false, pathToCS)
	
	ClearCase.mountMultipleVobs(configMap["requiredVobs"])
	
	String branch = "@@\\main\\branch_name"
	
	String pathToCSOnVob = configMap["configSpecOnView"].replace("%VIEWNAME%", viewNameCS)
	
	ccVersion = Integer.valueOf(ClearCase.getCcVersion(pathToCSOnVob + branch + "\\LATEST"))
	
	String[] ccLabel = ClearCase.getCcLabels(pathToCSOnVob + branch + "\\" + ccVersion)


	if (ccLabel.length == 0 || (ccLabel.length == 1 && ccLabel[0].trim().length() == 0)){
		ZfReport.warning("No new version avalaible. Nothing to build.")
		ClearCase.removeView(viewNameCS)
		System.exit(9)
	}

	//Load last builded version from lastJenkins.txt file
	myXmlStorage = new XmlStorage(Paths.get(configMap['resultDir']).resolve(configMap['lastJenkins']).toString(), false)
	
	try {
		myXmlStorage.loadData()
		lastVersion = myXmlStorage.getValue("version")
	} catch (Exception e) {
		lastVersion = -1
	}
	
	if (System.getenv("force").equals("true")) {
		lastVersion = -1
	}
	
	//Compare clearcase version against last builded version. Exit if clearcase version is lower or same as last builded version.
	if (ccVersion<=lastVersion){
		ZfReport.warning("No new version avalaible. Nothing to build.")
		ClearCase.removeView(viewNameCS)
		System.exit(9)
	}
	
	//Get current time in format for configspec rule
	Date dNow = new Date( )
	SimpleDateFormat ft = new SimpleDateFormat ("dd-MMM-yy.HH:mm:ss")
	String csTime = ft.format(dNow)
	
	
	String pathToNewCS = pathToCSOnVob + branch + "\\" + ccVersion
	//Parse creation time
	String creationTime = ClearCase.getCcDate(pathToNewCS)
	//Parse developer z number
	String developer = ClearCase.getCcCreatorNumber(pathToNewCS)
	//Parse developer name
	String developerName = ClearCase.getCcCreatorName(pathToNewCS)
	//Parse change request number 
	String crNumber = ClearCase.getCcCr(pathToNewCS)

	//Change format of the creation date
	creationTime = dNow.format("yyyy-MM-dd.HH-mm-ss")
	String creationTimeXML = "${creationTime}.${developer}.${crNumber}"

	//Saves data for build jobs
	myXmlStorage.setValue("creationTime", creationTimeXML)
	myXmlStorage.setValue("developer", developer)
	myXmlStorage.setValue("developerName", developerName)
	myXmlStorage.setValue("crNumber", crNumber)
	myXmlStorage.setValue("version", ccVersion)
	
	
	def src = new File(pathToNewCS)
	def dst = new File(configMap['resultDir'] + configMap["configSpecTime"])
	dst.write(src.text)
	
	
	myXmlStorage.saveData()
	ClearCase.removeView(viewNameCS)

	return
	
} catch(Exception e) {
	ZfReport.fatal(e.message)
	ZfReport.fatal(e.stackTrace)
	try {
		ClearCase.removeView(viewNameCS)
	} catch (Exception ex) {}
	
	System.exit(1)
}

